/**
 * 
 */
package clime.messadmin.providers.lifecycle;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import clime.messadmin.filter.MessAdminRequestWrapper;
import clime.messadmin.filter.MessAdminResponseWrapper;
import clime.messadmin.model.Application;
import clime.messadmin.model.Request;
import clime.messadmin.model.RequestInfo;
import clime.messadmin.model.Server;
import clime.messadmin.model.Session;
import clime.messadmin.providers.spi.RequestExceptionProvider;
import clime.messadmin.providers.spi.RequestLifeCycleProvider;
import clime.messadmin.servletstats.Utils;

/**
 * Collects statistics on Servlets
 * @author C&eacute;drik LIME
 */
public class ServletStatsGatherer implements RequestLifeCycleProvider, RequestExceptionProvider {
	private Request request = new Request(null);

	/**
	 * 
	 */
	public ServletStatsGatherer() {
		super();
	}

	/**
	 * {@inheritDoc}
	 */
	public void requestInitialized(HttpServletRequest httpRequest,
			HttpServletResponse httpResponse, ServletContext servletContext) {
		// Application
		Application application = Server.getInstance().getApplication(servletContext);
		final String servletPath = httpRequest.getContextPath() + httpRequest.getServletPath();
		RequestInfo requestInfo = (RequestInfo) Utils.getPluginData(application).get(servletPath);
		if (requestInfo == null) {
			requestInfo = new RequestInfo(servletPath);
			Utils.getPluginData(application).put(servletPath, requestInfo);
		}
		request.requestInitialized(requestInfo, httpRequest, servletContext);
		// Session
		if (httpRequest.getSession(false) != null) {
			Session session = application.getSession(httpRequest.getSession(false).getId());
			requestInfo = (RequestInfo) Utils.getPluginData(session).get(servletPath);
			if (requestInfo == null) {
				requestInfo = new RequestInfo(servletPath);
				Utils.getPluginData(session).put(servletPath, requestInfo);
			}
			request.requestInitialized(requestInfo, httpRequest, servletContext);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void requestDestroyed(HttpServletRequest httpRequest,
			HttpServletResponse httpResponse, ServletContext servletContext) {
		// Application
		Application application = Server.getInstance().getApplication(servletContext);
		final String servletPath = httpRequest.getContextPath() + httpRequest.getServletPath();
		RequestInfo requestInfo = (RequestInfo) Utils.getPluginData(application).get(servletPath);
		if (requestInfo == null) {
			requestInfo = new RequestInfo(servletPath);
			Utils.getPluginData(application).put(servletPath, requestInfo);
		}
		request.requestDestroyed(requestInfo, (MessAdminRequestWrapper)httpRequest, (MessAdminResponseWrapper)httpResponse, servletContext);
		// Session
		if (httpRequest.getSession(false) != null) {
			Session session = application.getSession(httpRequest.getSession(false).getId());
			requestInfo = (RequestInfo) Utils.getPluginData(session).get(servletPath);
			if (requestInfo == null) {
				// session was created with this hit
				requestInfo = new RequestInfo(servletPath);
				Utils.getPluginData(session).put(servletPath, requestInfo);
				request.requestInitialized(requestInfo, httpRequest, servletContext);
			}
			request.requestDestroyed(requestInfo, (MessAdminRequestWrapper)httpRequest, (MessAdminResponseWrapper)httpResponse, servletContext);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void requestException(Exception e, HttpServletRequest httpRequest, HttpServletResponse httpResponse, ServletContext servletContext) {
		requestDestroyed(httpRequest, httpResponse, servletContext);
		// Application
		Application application = Server.getInstance().getApplication(servletContext);
		final String servletPath = httpRequest.getContextPath() + httpRequest.getServletPath();
		RequestInfo requestInfo = (RequestInfo) Utils.getPluginData(application).get(servletPath);
		if (requestInfo == null) {
			requestInfo = new RequestInfo(servletPath);
			Utils.getPluginData(application).put(servletPath, requestInfo);
		}
		request.requestException(requestInfo, e, (MessAdminRequestWrapper)httpRequest, (MessAdminResponseWrapper)httpResponse, servletContext);
		// Session
		if (httpRequest.getSession(false) != null) {
			Session session = application.getSession(httpRequest.getSession(false).getId());
			requestInfo = (RequestInfo) Utils.getPluginData(session).get(servletPath);
			if (requestInfo == null) {
				requestInfo = new RequestInfo(servletPath);
				Utils.getPluginData(session).put(servletPath, requestInfo);
			}
			request.requestException(requestInfo, e, (MessAdminRequestWrapper)httpRequest, (MessAdminResponseWrapper)httpResponse, servletContext);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public int getPriority() {
		// no need for a priority, really
		return 0;
	}

}
